
# q-expansion of newform 1568.2.i.c, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1568
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[1471, [1, 0]], [197, [1, 0]], [1473, [0, -1]]]
aps_data = [[0, 0], [-2, 2], [0, 0], [0, 0], [4, -4], [4, 0], [-2, 2], [0, -6], [0, -8], [2, 0], [-4, 4], [0, -10], [10, 0], [4, 0], [0, 4], [2, -2], [10, -10], [0, -8], [8, -8], [0, 0], [-6, 6], [0, 16], [-2, 0], [0, 18], [2, 0], [0, 0], [0, -4], [0, 16], [-10, 10], [6, 0], [-16, 0], [0, -6], [14, -14], [10, 0], [0, -6], [16, -16], [12, -12], [0, -4], [-20, 0], [0, -12], [0, 0], [12, 0], [0, 0], [-14, 14], [6, 0], [4, -4], [8, 0], [-24, 0], [-22, 22], [0, -20], [0, 6], [24, 0], [14, -14], [10, 0], [0, 2], [16, -16], [-12, 12], [0, -16], [-14, 14], [-6, 0], [10, -10], [-8, 0], [-10, 0], [-24, 24], [0, -26], [0, 34], [0, 12], [-18, 0], [12, -12], [-20, 0], [18, -18], [0, 16], [8, -8], [0, -6], [28, 0], [0, 20], [6, -6], [0, -28], [0, 2], [-2, 2], [-6, 0], [-18, 0], [-8, 8], [-22, 0], [0, 24], [0, 24], [-30, 0], [0, -22], [-28, 0], [24, 0], [0, 18], [12, -12], [-24, 24], [8, 0], [0, 8], [16, 0], [0, 20], [38, -38], [0, -14], [0, -30], [-20, 0], [-30, 30], [46, -46], [0, -6], [4, -4], [-14, 14], [18, 0], [0, -6], [-24, 24], [46, 0], [0, 48], [22, -22], [22, 0], [-22, 22], [-16, 0], [34, -34], [34, 0], [-12, 12], [0, 38], [12, 0], [24, -24], [-6, 0], [0, 36], [24, -24], [0, -22], [-6, 0], [0, -26], [0, -36], [-28, 0], [0, 8], [-4, 4], [16, 0], [0, -24], [10, 0], [0, 30], [-6, 0], [-24, 24], [-46, 46], [-20, 0], [26, -26], [-34, 0], [0, 2], [-8, 8], [0, 0], [24, -24], [52, 0], [36, 0], [-18, 18], [0, -26], [0, -48], [0, 14], [38, 0], [-16, 0], [0, 28], [-32, 32], [16, 0], [0, 40], [0, -18], [22, 0], [-40, 40], [0, 4], [-22, 0], [-8, 0], [0, 42], [-26, 26], [-36, 36], [40, -40], [8, -8], [-2, 0], [0, -4], [-24, 24], [36, 0], [0, 48], [6, -6], [28, -28], [-26, 0], [-16, 0], [54, -54], [32, 0], [0, -8], [0, 8], [18, 0], [-14, 0], [0, 34], [0, 0], [-60, 60], [-22, 22], [-6, 6], [0, 14], [-12, 12], [0, 2], [-24, 0], [0, 32], [0, -20], [-32, 32], [-6, 6], [18, -18], [0, -2], [6, 0], [0, -8], [-54, 54], [28, 0], [0, -56], [-22, 22], [-66, 0], [40, -40], [0, 4], [0, 12], [30, 0], [-2, 2], [0, 50], [-32, 0], [-16, 0], [0, -54], [40, -40], [0, -2], [-40, 40], [54, -54], [0, -24], [10, 0], [0, -50], [-40, 0], [0, 30], [0, -24], [-26, 0], [-62, 0], [0, 50], [-40, 40], [0, 0], [0, 4], [6, -6], [-42, 42], [8, 0], [22, -22], [-2, 0], [-32, 32], [0, 30], [0, 42], [-8, 0], [32, 0], [36, -36], [0, 18], [-48, 48], [0, 10], [-26, 0], [0, -28], [-16, 0], [-6, 6], [60, -60], [24, 0], [-2, 0], [0, 34], [16, -16], [-50, 0], [-56, 56], [0, -64], [14, -14], [62, -62], [-64, 0], [0, -18], [24, -24], [-28, 0], [16, -16], [56, 0], [-50, 50], [0, -10], [14, 0], [-18, 0], [-48, 0], [-66, 66], [0, 36], [-76, 76], [-38, 38], [0, 64], [14, 0], [0, -16], [0, 60], [50, -50], [0, 2], [0, 50], [56, -56], [-40, 40], [20, 0], [8, 0], [0, 2], [0, 0], [46, -46], [38, 0], [-52, 52], [30, 0], [-70, 70], [18, -18], [0, -6], [46, 0], [22, 0], [-44, 44], [0, 80], [12, 0], [0, 38], [-46, 0], [0, 46], [0, 6], [56, -56]]
